#include <DigitalFilter.h>

dhpFilter::dhpFilter(long aInitValue) {
	initialize(aInitValue);
}

void dhpFilter::initialize(long aInitValue) {
	iLastSample = aInitValue;
	iShiftedFilter = aInitValue << 8;
}


long dhpFilter::value(long aSample) {

	long shiftedFCL = iShiftedFilter + ((aSample-iLastSample)<<8);

	iLastSample = aSample;
	iShiftedFilter = shiftedFCL - 5*(shiftedFCL>>7);
	return (iShiftedFilter+128)>>8;
}



dlpFilter::dlpFilter(long aInitValue) {
	initialize(aInitValue);
}

void dlpFilter::initialize(long aInitValue) {
	iLastSample = aInitValue;
	iShiftedFilter = aInitValue << DLP_FILTER_FILTERSHIFT;
}

long dlpFilter::value(long aSample) {

	iShiftedFilter += (aSample-iLastSample); // update the filter
	iLastSample = aSample;
	return ((iShiftedFilter+DLP_FILTER_FILTERROUNDING)>>DLP_FILTER_FILTERSHIFT);
}

avgFilter::avgFilter(int aSamples) {
	iSamples = (aSamples > 0 || aSamples <= DIGITALFILTER_NUMSAMPLES) ? aSamples : DIGITALFILTER_NUMSAMPLES;
	initialize();
}

void avgFilter::initialize() {
	iIndex = 0;
	iTotal = 0;
	iCount = 0;
	for (int i=0; i<iSamples; i++) iReadings[i]=0;
}

long	avgFilter::value(long aSample) {
	iTotal -= iReadings[iIndex];
	iReadings[iIndex] = aSample;
	iTotal += aSample;
	if (++iIndex >= iSamples) iIndex = 0;
	if (++iCount > iSamples) iCount = iSamples;
	return (iTotal/iCount);
}
